#
# COPYRIGHT (c) 2002 - 2013 SRON (R.M.van.Hees@sron.nl)
#
#   This is free software# you can redistribute it and/or modify it
#   under the terms of the GNU General Public License, version 2, as
#   published by the Free Software Foundation.
#
#   The software is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY# without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program# if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, 
#   Boston, MA  02111-1307, USA.
#
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
ls-option = $(shell if ls $(1) /dev/null 2>1; \
	      then echo $(1); else echo $(2); fi)

# include system dependent stuff
ifneq ($(_DEFINED_NADC_RULES),yes)
include $(TOPDIR)/config/Rules_NADC.mk
endif
export HPATH

ifndef IDL_LINK
	IDL_LINK = 0
endif
ifndef ADDLIBS
	ADDLIBS = 
endif
ifndef ADDOBJS
	ADDOBJS = $(OBJS)
endif

ifeq ($(SHARED_LIBS),yes)
	CFLAGS += $(SH_OPT)
endif

# add additional include files, for example HDF5
ADD_INC += $(HDF5_INC)
ifeq ($(IDL_LINK),1)
find_lz = $(shell if [ -r $(HDF5_DIR)/lib/libz.a ]; \
	then echo "$(HDF5_DIR)/lib/libz.a"; else echo "/usr/lib/libz.a"; fi)

ADD_INC += $(IDL_INC)
LDFLAGS += -Wl,-Bsymbolic
ifeq ($(WITH_CC),gnu)
LDFLAGS += -Wl,--unresolved-symbols=ignore-in-shared-libs
endif
LLIBS	= $(HDF5_DIR)/lib/libhdf5_hl.a $(HDF5_DIR)/lib/libhdf5.a
LLIBS	+= $(call find_lz) -ldl -lm
endif

VERS_DATE = $(shell ls -ltr $(call cc-option,--time-style=long-iso,"") | \
	      tail -1 | awk '{print $$6}')
VERS_LIB = $(VERS_MAJOR).$(VERS_MINOR).$(VERS_RELEASE)
SH_LIB	= $(LIBNAME).$(SO_EXT).$(VERS_LIB)
SH_LINK	= $(LIBNAME).$(SO_EXT)

# Rules:
.PHONY:	all clean realclean distclean
all:	libs

#%.d:	%.c version.h
#	@set -e; rm -f $@; \
#	$(CC) -MM -I$(INCPATH) $(ADD_INC) $(CPPFLAGS) $< > $@.$$$$; \
#	sed 's,\($*\)\.o[ :]*,\1.o $@ : ,g' < $@.$$$$ > $@; \
#	rm -f $@.$$$$

version.h:	.version

.PHONY:	depend dep
.depend:
	$(CC) -MM -I$(INCPATH) $(ADD_INC) $(subst .o,.c,$(OBJS)) > .depend

depend dep:	.depend

libs:	depend $(OBJS)
	$(MKDIR) $(LIBPATH)
ifeq ($(SHARED_LIBS),yes)
	$(SH_LD) $(LDFLAGS) -o $(LIBPATH)/$(SH_LIB) $(ADDOBJS) $(ADDLIBS)
	(cd $(LIBPATH); $(RM) $(SH_LINK); $(LN) $(SH_LIB) $(SH_LINK))
else
	$(AR) $(AR_OPT) $(LIBPATH)/$(AR_LIB) $(OBJS)
endif

install:	libs
	$(MKDIR) $(LIBDIR)
ifeq ($(SHARED_LIBS),yes)
	$(INSTALL) $(LIBPATH)/$(SH_LIB) $(LIBDIR)
	(cd $(LIBDIR); $(RM) $(SH_LINK); $(LN) $(SH_LIB) $(SH_LINK))
else
	$(INSTALL) $(LIBPATH)/$(AR_LIB) $(LIBDIR)
endif

clean:
ifdef $(SUBLIBDIR)
	@for i in $(SUBLIBDIR) ; do \
	        $(MAKE) -C $$i clean ; \
	        if [ $$? -ne 0 ] ; then break ; fi ; \
	done
endif
	$(RM) $(LIBPATH)/$(AR_LIB) $(LIBPATH)/$(SH_LIB) $(OBJS)

realclean:	clean
ifdef $(SUBLIBDIR)
	@for i in $(SUBLIBDIR) ; do \
	        $(MAKE) -C $$i realclean ; \
	        if [ $$? -ne 0 ] ; then break ; fi ; \
	done
endif
	(cd $(LIBPATH); $(RM) $(SH_LIB) $(SH_LINK))
	(cd $(LIBPATH); $(RM) $(AR_LIB))
	$(RM) .depend .version $(TEST_PROGS)

distclean:	realclean
	(cd $(LIBDIR); $(RM) $(SH_LIB) $(SH_LINK))
	(cd $(LIBDIR); $(RM) $(AR_LIB))

#+++++++++++++++++++++++++ RULES:
.SUFFIXES:
.SUFFIXES: .o .c .f .for .cc
.c.o:
	$(CC) -c $(CFLAGS) -I$(INCPATH) $(ADD_INC) -o $@ $<
.cc.o:
	$(CXX) -c $(CXXFLAGS) -I$(INCPATH) $(ADD_INC) -o $@ $<
.for.o:
	$(F77) -c $(FFLAGS) -I$(INCPATH) $(ADD_INC) -o $@ $<
.f.o:
	$(F77) -c $(FFLAGS) -I$(INCPATH) $(ADD_INC) -o $@ $<
