#ifndef  __DEFS_MCFS                            /* Avoid redefinitions */
#define  __DEFS_MCFS

#ifdef __cplusplus
extern "C" {
#endif

#define Title \
   "Meris cloud fraction for Sciamachy pixels"

#define ADAGUC_REF_DOC         "ADAGUC Data Products Standard"
#define ADAGUC_REF_VERSION     "1.1"

#define ADAGUC_PROD_TEMPLATE   "MERIS_CONS_V___CLOUDFR__L2__%s_%s_%04d.nc"

#define Copyright "TBD"

#define Abstract  "The Sciamachy/Meris cloud fraction synergy product was"\
     " developed by the University of Bremen within ESA's G-POD Category-1"\
     " Opportunity framework (http://eopi.esa.int/G-POD)."\
     " The cloud fraction is an important atmospheric quantity needed for "\
     " an accurate determination of trace gas concentration as well as cloud"\
     " and aerosol properties from space-borne optical measurements. The"\
     " retrieval algorithm is described here."\
     " The products have been processed on G-POD for the whole Envisat"\
     " mission (2002-present)."\
     "The results are organised per month and contain the cloud fraction"\
     " data (in MCFS ASCII files) corresponding to each Sciamachy"\
     " consolidated level 1 product distributed by ESA."

#define Reference "http://eogrid.esrin.esa.int/index.asp"

#define MCFS_NC_VARS "cloudFraction,cloudFractionThick,tile"

#ifdef __NEED_ISO_ENTRIES
struct iso_entry_t {
     const char *attr_name;
     const char *attr_value;
};

static struct iso_entry_t meta_root_list[] = {
     { "title", Title },
     { "institution", "IUP-Bremen" },
     { "source", NULL },
     { "history", "none" },
     { "references", Reference },
     { "comment", "none" },
     { "Convention", "CF-1.4"}
};
/*@unused@*/
static const unsigned short numRootKeys = (unsigned short)
     (sizeof(meta_root_list) / sizeof(struct iso_entry_t));

static struct iso_entry_t meta_prod_list[] = {
     { "ref_doc", ADAGUC_REF_DOC },
     { "ref_doc_version", ADAGUC_REF_VERSION },
     { "format_version", "1.0" },
     { "input_products", NULL },
     { "originator", "G-POD (ESA)" },
     { "creation_date", NULL },
     { "software_version", NULL },
     { "file_class", "CONS" },
     { "type", "V" },
     { "acronym", "MCFS" },
     { "level", "L2" },
     { "variables", MCFS_NC_VARS },
     { "validity_start", NULL },
     { "validity_stop", NULL },
     { "style", "lowercase" }
};
static const unsigned short numProdKeys = 
     (sizeof(meta_prod_list) / sizeof(struct iso_entry_t));

static struct iso_entry_t geo_prod_list[] = {
     { "projection_name", "Latitude Longitude" },
     { "EPSG_code", "EPSG:4326" },
     { "proj4_params", "+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs" },
     { "grid_mapping_name", "latitude_longitude" }
};
static const unsigned short numGeoKeys = 
     (sizeof(geo_prod_list) / sizeof(struct iso_entry_t));

static const struct iso_entry_t iso_prod_list[] = {
     { "title", Title },
     { "abstract", Abstract },
     { "status", "completed" },
     { "type", "dataset" },
     { "uid", "                                " },
     { "topic", "atmosphere" },
     { "keyword", "Tropospheric Atmospheric Global " \
       "[Theme=atmosphere,air quality],[Place=Global Statum=troposphere]," \
       "[Temporal=recent years]" },
     { "keyword_title", "netCDF-CF" },
     { "max-x", "180" },
     { "min-x", "-180" },
     { "max-y", "90" },
     { "min-y", "-90" },
     { "temporal_extend", "2002-09-01 2009-01-01" },
     { "date", "2009-01-24" },
     { "dateType", "revision" },
     { "statement", "Is eene bepaalde oorzaak gegeven, dan volgt" \
       " noodzakelijk het uitwerksel; en omgekeerd," \
       " indien er geene bepaalde oorzaak gegeven is, dan" \
       " is het onmogelijk, dat het uitwerksel volgt. (Spinoza)" },
     { "code", "4326 (WGS84)" },
     { "codeSpace", "EPSG" },
     { "accessConstraints", Copyright },
     { "useLimitation", "inapplicable" },
     { "organisationName_dataset",
       "IUP University of Bremen" },
     { "email_dataset", "http://www.iup.uni-bremen.de/eng/" },
     { "role_dataset", "pointOfContact" },
     { "metadata-id", "                                " },
     { "organisationName_metadata",
       "ESA Earth Observation GRID Processing-on-Demand (GPOD)" },
     { "role_metadata", "pointOfContact" },
     { "email_metadata", "adaguc@knmi.nl" },
     { "ulr_metadata", "http://adaguc.knmi.nl" },
     { "datestamp", "2009-01-24" },
     { "language", "eng" },
     { "metadataStandardName", "ISO19115" },
     { "metadataStandardNameVersion",
       "Nederlandse metadatastandaard voor geografie 1.2" }
};
static const unsigned short numIsoKeys = 
     (sizeof(iso_prod_list) / sizeof(struct iso_entry_t));
#endif /* __NEED_ISO_ENTRIES */

/* SCIA/MERIS MCFS structures */
struct mcfs_hdr {
     char  product[ENVI_FILENAME_SIZE+6];
     char  institution[8];
     char  contact[64];
     char  creation_date[MAX_UTC_STRING];
     char  receive_date[MAX_UTC_STRING];
     char  l1b_product[ENVI_FILENAME_SIZE];
     char  product_datetime_start[MAX_UTC_STRING];
     char  product_datetime_end[MAX_UTC_STRING];
     unsigned short numRec;
     unsigned int   orbit;
     unsigned int   file_size;
};

struct mcfs_meta_rec {
     unsigned char intg_time;
     unsigned char quality;
     unsigned short numMeris;
     unsigned short numPMD;
     float sun_zen_ang;
     float sun_azi_ang;
     float sat_zen_ang;
     float sat_azi_ang;
};

struct mcfs_rec
{
     char date[12];
     char time[14];
     double jday;
     unsigned int orbit;
     float cloudFraction;
     float cloudFractionThick;
     struct mcfs_meta_rec meta;
     struct coord_envi center;
};

/* SCIA/MERIS MCFS fuction prototypes */
extern void NADC_RD_MCFS( const char *, /*@out@*/ struct mcfs_hdr *hdr, 
			  /*@null@*/ /*@out@*/ struct mcfs_rec **rec )
      /*@globals  errno, nadc_stat, nadc_err_stack;@*/
      /*@modifies errno, nadc_stat, nadc_err_stack, hdr, rec@*/;

#if defined(_NETCDF_)
extern void NADC_MCFS_WR_NC_META( int, const struct mcfs_hdr * );
extern void NADC_MCFS_WR_NC_REC( int, unsigned short, const struct mcfs_rec * );
#endif

#ifdef _KNMI_SQL
#if defined(LIBPQ_FE_H)
extern void NADC_MCFS_DEL_ENTRY( PGconn *conn, const char * )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, conn@*/;
extern void NADC_MCFS_WR_SQL_META( PGconn *conn, const struct mcfs_hdr * )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, conn@*/;
extern void NADC_MCFS_WR_SQL_TILE( PGconn *conn, const char *, unsigned short,
                                   const struct mcfs_rec * )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, conn@*/;
#endif
#endif

#ifdef __cplusplus
  }
#endif
#endif   /* __DEFS_MCFS */
