/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT (c) 2003 - 2013 SRON (R.M.van.Hees@sron.nl)

   This is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License, version 2, as
   published by the Free Software Foundation.

   The software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA  02111-1307, USA.

.IDENTifer   PROTO_SCIA_LV0
.AUTHOR      R.M. van Hees 
.KEYWORDS    header file
.LANGUAGE    ANSI-C
.PURPOSE     prototypes of the specific SCIA level 0 modules
.ENVIRONment none
.VERSION     2.0     01-Nov-2001   moved to new Error handling routines, RvH
             1.0     13-Aug-2001   Creation by R.M. van Hees
------------------------------------------------------------*/

#ifndef  NADC_PROTO_SCIA_LV0                       /* Avoid redefinitions */
#define  NADC_PROTO_SCIA_LV0

#if defined _STDIO_INCLUDED || defined _STDIO_H || defined __STDIO_H__
extern void SCIA_LV0_RD_SPH( FILE *fd, const struct mph_envi,
			     /*@out@*/ struct sph0_scia *sph )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd, *sph@*/;
extern void SCIA_LV0_WR_SPH( FILE *fd, const struct mph_envi,
			     const struct sph0_scia sph )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd@*/;

extern unsigned int SCIA_LV0_RD_MDS_INFO( FILE *fd, unsigned int, 
					  const struct dsd_envi *,
					  /*@out@*/ struct mds0_info **info )
       /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd, *info@*/;

extern unsigned int GET_SCIA_LV0_MDS_INFO( FILE *fd, struct mph_envi,
					   const struct dsd_envi *, 
					   struct mds0_info *info )
       /*@globals  errno, stderr, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, stderr, nadc_stat, nadc_err_stack, fd, info@*/;

extern unsigned int SCIA_LV0_RD_AUX( FILE *fd, const struct mds0_info *,
                                     unsigned int,
                                     /*@out@*/ struct mds0_aux **aux_out )
       /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd, *aux_out@*/;
extern unsigned int SCIA_LV0_WR_AUX( FILE *fd, const struct mds0_info *,
                                     unsigned int, const struct mds0_aux * )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd@*/;
extern unsigned int GET_SCIA_LV0_STATE_AUX( FILE *, const struct mds0_info *,
				     unsigned int, struct mds0_aux **aux_out )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd, *aux_out@*/;

extern unsigned int SCIA_LV0_RD_DET( FILE *fd, const struct mds0_info *,
                                     unsigned int, unsigned char,
                                     /*@out@*/ struct mds0_det **det_out )
       /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd, *det_out@*/;

extern unsigned int SCIA_LV0_WR_DET( FILE *fd, const struct mds0_info *,
                                     unsigned int, const struct mds0_det * )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd@*/;
extern unsigned int GET_SCIA_LV0_STATE_DET( unsigned char, FILE *, 
					    const struct mds0_info *,
					    unsigned int, 
					    struct mds0_det **det_out )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd, det_out@*/;

extern unsigned int SCIA_LV0_RD_PMD( FILE *fd, const struct mds0_info *,
                                     unsigned int,
                                     /*@out@*/ struct mds0_pmd **pmd_out )
       /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd, *pmd_out@*/;
extern unsigned int SCIA_LV0_WR_PMD( FILE *fd, const struct mds0_info *,
                                     unsigned int, const struct mds0_pmd * )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd@*/;
extern unsigned int GET_SCIA_LV0_STATE_PMD( FILE *, const struct mds0_info *,
				     unsigned int, struct mds0_pmd **pmd_out )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd, pmd_out@*/;

extern void SCIA_LV0_RD_LV1_AUX( FILE *fd, /*@out@*/ struct mds0_aux *aux )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd, *aux@*/;
extern unsigned int SCIA_LV0_WR_LV1_AUX( FILE *fd, const struct mds0_aux )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd@*/;

extern void SCIA_LV0_RD_LV1_PMD( FILE *fd, /*@out@*/ struct mds0_pmd *pmd )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd, *pmd@*/;
extern unsigned int SCIA_LV0_WR_LV1_PMD( FILE *fd, const struct mds0_pmd )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fd@*/;

extern void SCIA_WR_ASCII_LV0_AUX( FILE *fd, unsigned int,
				   const struct mds0_aux * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack, fd@*/;
extern void SCIA_WR_ASCII_LV0_PMD( FILE *fd, unsigned int,
				   const struct mds0_pmd * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack, fd@*/;
#endif   /* ---- defined _STDIO_INCLUDED || defined _STDIO_H ----- */

extern double GET_SCIA_LV0_MDS_TIME( int, const void * );
extern void   GET_SCIA_LV0_MDS_TIME_ARR(int, const void *, /*@out@*/ double *);
extern void   GET_SCIA_LV0_MDS_ANGLES( const struct mds0_aux *, 
				       /*@out@*/ float *, /*@out@*/ float * );
extern void   GET_SCIA_LV0_STATE_ANGLE( unsigned short, 
					const struct mds0_aux *, 
					/*@out@*/ float *, /*@out@*/ float * );
extern void   GET_SCIA_LV0_STATE_OBMtemp( bool, unsigned short, 
					  const struct mds0_aux *, 
					  /*@out@*/ float * );
extern void   GET_SCIA_LV0_STATE_DETtemp( unsigned short, 
					  const struct mds0_det *,
					  /*@out@*/ float * );
extern void   GET_SCIA_LV0_STATE_PMDtemp( unsigned short, 
					  const struct mds0_pmd *, 
					  /*@out@*/ float * );
extern void GET_SCIA_LV0_DET_PET( struct chan_hdr, 
				  /*@out@*/ float *, 
				  /*@out@*/ unsigned short * );
extern unsigned short GET_SCIA_LV0C_MDS( unsigned int, 
					 const struct mds0_info *,
					 const struct mds0_det *, 
					 /*@out@*/ struct mds1c_scia **mds )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack, *mds@*/;

extern void SCIA_LV0_WR_ASCII_SPH( struct param_record, 
				   const struct sph0_scia * )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack@*/;

extern void SCIA_LV0_WR_ASCII_INFO( struct param_record, unsigned int,
				    const struct mds0_info * )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack@*/;

extern void SCIA_LV0_FREE_MDS_DET( unsigned int, 
                                   /*@only@*/ struct mds0_det * );

extern unsigned int SCIA_LV0_SELECT_MDS( int, const struct param_record,
					 unsigned int, 
					 const struct mds0_info *,
			  /*@null@*/ /*@out@*/ struct mds0_info **info )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, *info@*/;

extern void SCIA_LV0_WR_ASCII_AUX( struct param_record, unsigned int,
				   unsigned int, const struct mds0_aux * )
      /*@globals  errno, nadc_stat, nadc_err_stack;@*/
      /*@modifies errno, nadc_stat, nadc_err_stack@*/;
 
extern void SCIA_LV0_WR_ASCII_DET( struct param_record, unsigned int,
				   unsigned int, const struct mds0_det * )
      /*@globals  nadc_stat, nadc_err_stack;@*/
      /*@modifies nadc_stat, nadc_err_stack@*/;
 
extern void SCIA_LV0_WR_ASCII_PMD( struct param_record, unsigned int,
				   unsigned int, const struct mds0_pmd * )
      /*@globals  nadc_stat, nadc_err_stack;@*/
      /*@modifies nadc_stat, nadc_err_stack@*/;
 
#ifdef _HDF5_H
extern void SCIA_LV0_WR_H5_SPH( struct param_record, 
				const struct sph0_scia * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack*/;
extern void SCIA_LV0_WR_H5_AUX( struct param_record, unsigned int, 
				unsigned int, const struct mds0_aux * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack*/;
extern void SCIA_LV0_WR_H5_DET( struct param_record, const struct mds0_info *, 
				unsigned int, const struct mds0_det * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack*/;
extern void SCIA_LV0_WR_H5_PMD( struct param_record, unsigned int, 
				unsigned int, const struct mds0_pmd * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack*/;
extern unsigned int SCIA_LV0_RD_H5_INFO_DB( const char *,
					    struct mds0_info **info )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, *info@*/;
#endif /* _HDF5_H */

#endif /* NADC_PROTO_SCIA_LV0 */
