/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT (c) 2001 - 2013 SRON (R.M.van.Hees@sron.nl)

   This is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License, version 2, as
   published by the Free Software Foundation.

   The software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, 
   Boston, MA  02111-1307, USA.

.IDENTifer   UNPACK_LV0_DATA
.PURPOSE     unpack 16/24 bit detector data as 32 bit values
.INPUT/OUTPUT
  call as   UNPACK_LV0_PIXEL_VAL( pixel, data )
     input:  
             struct chan_src *pixel :  detector source packet
    output:  
             unsigned int *data     :  detector pixel data

.RETURNS     nothing
.COMMENTS    ANSI-C inline function, declare in your code as:
             #include <unpack_lv0_data.inc>
.VERSION      1.3   23-Apr-2002 do not need to byte-swap (Thanks Jochen), RvH
              1.2   19-Mar-2002	forgot to byte-swap data, RvH
              1.1   01-Feb-2002	wrote documentation, RvH 
              1.0   18-Dec-2001	Created by R. M. van Hees 
------------------------------------------------------------*/
#ifndef  __LV0_UNPACK                            /* Avoid redefinitions */
#define  __LV0_UNPACK

static inline
void UNPACK_LV0_PIXEL_VAL( const struct chan_src *pixel,
                           /*@out@*/ unsigned int *data )
{
     register unsigned short np = 0;

     register unsigned char *cpntr = pixel->data;

     if ( pixel->co_adding == (unsigned char) 1 ) {
	  do {
	       *data++ = (unsigned int) cpntr[1]
		    + ((unsigned int) cpntr[0] << 8);
	       cpntr += 2;
	  } while ( ++np < pixel->length );
     } else {
	  do {
	       *data++ = (unsigned int) cpntr[2]
		    + ((unsigned int) cpntr[1] << 8)
		    + ((unsigned int) cpntr[0] << 16);
	       cpntr += 3;
	  } while ( ++np < pixel->length );
     }
}
#endif
