/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT (c) 2007 - 2013 SRON (R.M.van.Hees@sron.nl)

   This is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License, version 2, as
   published by the Free Software Foundation.

   The software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, 
   Boston, MA  02111-1307, USA.

.IDENTifer   SCIA_DEL_SQL_DMOP
.AUTHOR      R.M. van Hees
.KEYWORDS    SCIA DMOP SQL
.LANGUAGE    ANSI C
.PURPOSE     remove entries in database for a Sciamachy DMOP product (SOST)
.INPUT/OUTPUT
  call as   SCIA_DEL_SQL_DMOP( conn, numRec, dmop )
     input:  
             PGconn *conn        :  PostgreSQL connection handle
	     unsigned int numRec :   number of DMOP records
	     struct dmop_rec *dmop : records with DMOP data	     

.RETURNS     Nothing, error status passed by global variable ``nadc_stat''
.COMMENTS    Removes entries from tables: stateinfo
.ENVIRONment None
.VERSION     1.1     12-Jan-2012   remove all entries, 
                                   even when an update took place, RvH
             1.0     07-Jun-2007   initial release by R. M. van Hees
------------------------------------------------------------*/
/*
 * Define _ISOC99_SOURCE to indicate
 * that this is a ISO C99 program
 */
#define  _ISOC99_SOURCE

/*+++++ System headers +++++*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <libpq-fe.h>

/*+++++ Local Headers +++++*/
#define _SCIA_COMMON
#include <defs_nadc.h>

/*+++++ Macros +++++*/
#define SQL_STR_SIZE   128

#define DELETE_FROM_STATEINFO \
"DELETE FROM stateinfo WHERE dateTimeStart >=\'%s\' AND dateTimeStart <=\'%s\'"

/*+++++ Global Variables +++++*/
	/* NONE */

/*+++++ Static Variables +++++*/
	/* NONE */

/*+++++++++++++++++++++++++ Static Functions +++++++++++++++++++++++*/
				/* NONE */

/*+++++++++++++++++++++++++ Main Program or Function +++++++++++++++*/
void SCIA_DEL_SQL_DMOP( PGconn *conn, unsigned int numRec, 
			const struct dmop_rec *dmop )
{
     const char prognm[] = "SCIA_DEL_SQL_DMOP";

     char sql_query[SQL_STR_SIZE];

     PGresult *res_del;
/*
 * remove entries from table "stateinfo"
 */
     (void) snprintf( sql_query, SQL_STR_SIZE, DELETE_FROM_STATEINFO,
		      dmop[0].dateTimeStart, dmop[numRec-1].dateTimeStop );
/*    (void) fprintf( stderr, "%s\n", sql_query ); */
     res_del = PQexec( conn, sql_query );
     if ( PQresultStatus( res_del ) != PGRES_COMMAND_OK )
	  NADC_ERROR( prognm, NADC_ERR_SQL, PQresultErrorMessage(res_del) );
     PQclear( res_del );
}
