/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT (c) 2007 - 2013 SRON (R.M.van.Hees@sron.nl)

   This is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License, version 2, as
   published by the Free Software Foundation.

   The software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA  02111-1307, USA.

.IDENTifer   SCIA_WR_SQL_DMOP
.AUTHOR      R.M. van Hees
.KEYWORDS    SCIA PostgreSQL
.LANGUAGE    ANSI C
.PURPOSE     write 
.INPUT/OUTPUT
  call as   SCIA_WR_SQL_DMOP( conn, numRec, dmop );
     input:
             PGconn *conn  :         PostgreSQL connection handle
	     unsigned int numRec :   number of DMOP records
	     struct dmop_rec *dmop : records with DMOP data

.RETURNS     Nothing, error status passed by global variable ``nadc_stat''
.COMMENTS    None
.ENVIRONment None
.VERSION     1.0     04-Jun-2007   initial release by R. M. van Hees
------------------------------------------------------------*/
/*
 * Define _ISOC99_SOURCE to indicate
 * that this is a ISO C99 program
 */
#define  _ISOC99_SOURCE

/*+++++ System headers +++++*/
#include <stdio.h>
#include <string.h>

#include <libpq-fe.h>

/*+++++ Local Headers +++++*/
#define _SCIA_COMMON
#include <defs_nadc.h>

/*+++++ Macros +++++*/
#define SQL_STR_SIZE   512

#define INSERT_TO_STATEINFO \
"INSERT INTO stateinfo (dateTimeStart,muSecStart,dateTimeStop,muSecStop,\
timeLine,stateID,absOrbit,orbitPhase,tile) VALUES"

#define TILE_DEFAULT \
"ST_GeomFromText(\'POLYGON((0 0, 0 0, 0 0, 0 0, 0 0))\',4326)"

/*+++++ Global Variables +++++*/
        /* NONE */

/*+++++ Static Variables +++++*/
        /* NONE */

/*+++++++++++++++++++++++++ Static Functions +++++++++++++++++++++++*/

/*+++++++++++++++++++++++++ Main Program or Function +++++++++++++++*/
void SCIA_WR_SQL_DMOP( PGconn *conn, unsigned int numRec, 
		       const struct dmop_rec *dmop )
{
     const char prognm[] = "SCIA_WR_SQL_DMOP";

     register unsigned int nr;

     char sql_query[SQL_STR_SIZE], cbuff[SQL_STR_SIZE];

     int  numChar;

     PGresult *res;
/*
 * Start a transaction block
 */
     res = PQexec( conn, "BEGIN" );
     if ( PQresultStatus( res ) != PGRES_COMMAND_OK ) {
	  NADC_GOTO_ERROR( prognm, NADC_ERR_SQL, PQresultErrorMessage(res) );
     }
     PQclear( res );
/*
 * create insert-query
 */
     for ( nr = 0; nr < numRec; nr++ ) {
	  (void) strcpy( sql_query, INSERT_TO_STATEINFO );
/* dateTimeStart */
	  (void) snprintf( sql_query, SQL_STR_SIZE,
			   "%s (\'%s\',", strcpy(cbuff,sql_query), 
			   dmop[nr].dateTimeStart );
/* muSecStart */
	  (void) snprintf( sql_query, SQL_STR_SIZE,
			   "%s%6u,", strcpy(cbuff,sql_query),
			   dmop[nr].muSecStart );
/* dateTimeStop */
	  (void) snprintf( sql_query, SQL_STR_SIZE,
			   "%s\'%s\',", strcpy(cbuff,sql_query),
			   dmop[nr].dateTimeStop );
/* muSecStop */
	  (void) snprintf( sql_query, SQL_STR_SIZE,
			   "%s%6u,", strcpy(cbuff,sql_query), 
			   dmop[nr].muSecStop );
/* timeLine */
	  (void) snprintf( sql_query, SQL_STR_SIZE, "%s\'%s\',",
			   strcpy(cbuff,sql_query), dmop[nr].timeLine );
/* stateID */
	  (void) snprintf( sql_query, SQL_STR_SIZE, "%s%hhu,",
			   strcpy(cbuff,sql_query), dmop[nr].stateID );
/* absOrbit */
	  (void) snprintf( sql_query, SQL_STR_SIZE, "%s%hu,",
			   strcpy(cbuff,sql_query), dmop[nr].absOrbit );
/* orbitPhase */
          (void) snprintf( sql_query, SQL_STR_SIZE, "%s %.5f,",
			   strcpy(cbuff,sql_query), dmop[nr].orbitPhase );
/* tile */
	  numChar = snprintf( sql_query, SQL_STR_SIZE, "%s%s)",
			      strcpy(cbuff,sql_query), TILE_DEFAULT );
/* 	  (void) fprintf( stderr, "%s [%-d]\n", sql_query, numChar ); */
	  if ( numChar >= SQL_STR_SIZE ) {
	       res = PQexec( conn, "ROLLBACK" );
	       NADC_GOTO_ERROR( prognm, NADC_ERR_STRLEN,  "sql_query" );
	  }
/*
 * do the actual insert
 */
	  res = PQexec( conn, sql_query );
	  if ( PQresultStatus( res ) != PGRES_COMMAND_OK ) {
	       NADC_ERROR( prognm, NADC_ERR_SQL, PQresultErrorMessage(res) );
	       PQclear( res );
	       res = PQexec( conn, "ROLLBACK" );
	       if ( PQresultStatus( res ) != PGRES_COMMAND_OK )
		    NADC_ERROR(prognm,NADC_ERR_SQL,PQresultErrorMessage(res));
	       goto done;
	  }
	  PQclear( res );
     }
/*
 * end the transaction
 */
     res = PQexec( conn, "COMMIT" );
     if ( PQresultStatus( res ) != PGRES_COMMAND_OK ) {
	  NADC_GOTO_ERROR( prognm, NADC_ERR_SQL, PQresultErrorMessage(res) );
     }
 done:
     PQclear( res );
}
