DROP TABLE IF EXISTS auxiliary;

DROP TABLE IF EXISTS fresco_tileinfo;
DROP TABLE IF EXISTS imap_ch4_tileinfo;
DROP TABLE IF EXISTS imap_hdo_tileinfo;
DROP TABLE IF EXISTS imlm_co_tileinfo;
DROP TABLE IF EXISTS imlm_h2o_tileinfo;
DROP TABLE IF EXISTS mcfs_tileinfo;
DROP TABLE IF EXISTS tosomi_tileinfo;
DROP TABLE IF EXISTS cld_ol_tileinfo;
DROP TABLE IF EXISTS no2_ol_tileinfo;
DROP TABLE IF EXISTS o3_ol_tileinfo;

DROP TABLE IF EXISTS tile_fresco;
DROP TABLE IF EXISTS tile_imap_ch4;
DROP TABLE IF EXISTS tile_imap_hdo;
DROP TABLE IF EXISTS tile_imlm_co;
DROP TABLE IF EXISTS tile_imlm_h2o;
DROP TABLE IF EXISTS tile_mcfs;
DROP TABLE IF EXISTS tile_tosomi;
DROP TABLE IF EXISTS tile_cld_ol;
DROP TABLE IF EXISTS tile_no2_ol;
DROP TABLE IF EXISTS tile_o3_ol;

DROP TABLE IF EXISTS tileinfo;

DROP TABLE IF EXISTS stateinfo_meta__2P;
DROP TABLE IF EXISTS stateinfo_meta__1P;
DROP TABLE IF EXISTS stateinfo_meta__0P;

DROP TABLE IF EXISTS meta_fresco;
DROP TABLE IF EXISTS meta_imap_ch4;
DROP TABLE IF EXISTS meta_imap_hdo;
DROP TABLE IF EXISTS meta_imlm_co;
DROP TABLE IF EXISTS meta_imlm_h20;
DROP TABLE IF EXISTS meta_mcfs;
DROP TABLE IF EXISTS meta_tosomi;
DROP TABLE IF EXISTS meta__2P;
DROP TABLE IF EXISTS meta__1P;
DROP TABLE IF EXISTS meta__0P;

--
-- Table structure for table meta__0P
--
CREATE TABLE meta__0P (
  pk_meta        serial      PRIMARY KEY,
  name           char(62)    NOT NULL UNIQUE CHECK (name <> ''),
  fileSize       integer     NOT NULL CHECK (fileSize > 0),
  receiveDate    timestamp   NOT NULL,
  procStage      proc_stage  NOT NULL,
  procCenter     char(6)     NOT NULL,
  softVersion    varchar(14) NOT NULL,
  dateTimeStart  timestamp   NOT NULL,
  muSecStart     integer     NOT NULL CHECK (muSecStart >= 0),
  dateTimeStop   timestamp   NOT NULL,
  muSecStop      integer     NOT NULL CHECK (muSecStop >= 0),
  absOrbit       integer     NOT NULL CHECK (absOrbit >= 0),
  relOrbit       smallint    NOT NULL,
  numDataSets    smallint    NOT NULL,
  nadirStates    smallint    NOT NULL,
  limbStates     smallint    NOT NULL,
  occultStates   smallint    NOT NULL,
  monitorStates  smallint    NOT NULL,
  noEntryDMOP    smallint    DEFAULT 0,
  delayedBy      real        
);
-- CREATE INDEX meta__0P_absOrbit      ON meta__0P (absOrbit);
-- CREATE INDEX meta__0P_dateTimeStart ON meta__0P (dateTimeStart);

--
-- Table structure for table meta__1P
--
CREATE TABLE meta__1P (
  pk_meta          serial      PRIMARY KEY,
  name             char(62)    NOT NULL UNIQUE CHECK (name <> ''),
  fileSize         integer     NOT NULL CHECK (fileSize > 0),
  receiveDate      timestamp   NOT NULL,
  procStage        proc_stage  NOT NULL,
  procCenter       char(6)     NOT NULL,
  softVersion      varchar(14) NOT NULL,
  keydataVersion   varchar(8)  NOT NULL,
  mFactorVersion   varchar(8)  NOT NULL,
  spectralCal      quality     NOT NULL,
  saturatedPix     quality     NOT NULL,
  deadPixels       quality     NOT NULL,
  dateTimeStart    timestamp   NOT NULL,
  muSecStart       integer     NOT NULL CHECK (muSecStart >= 0),
  dateTimeStop     timestamp   NOT NULL,
  muSecStop        integer     NOT NULL CHECK (muSecStop >= 0),
  absOrbit         integer     NOT NULL CHECK (absOrbit >= 0),
  relOrbit         smallint    NOT NULL,
  numDataSets      smallint    NOT NULL,
  leakageFileID    integer    ,  
  ppgEtalonFileID  integer    ,  
  spectralFileID   integer    ,  
  sunRefFileID     integer    ,  
  keyDataFileID    integer    ,  
  mFactorFileID    integer    ,  
  initFileID       integer    ,  
  nadirStates      smallint    NOT NULL,
  limbStates       smallint    NOT NULL,
  occultStates     smallint    NOT NULL,
  monitorStates    smallint    NOT NULL,
  noProcStates     smallint    NOT NULL,
  noEntryDMOP      smallint    DEFAULT 0,
  delayedBy        real        
);
-- CREATE INDEX meta__1P_absOrbit      ON meta__1P (absOrbit);
-- CREATE INDEX meta__1P_dateTimeStart ON meta__1P (dateTimeStart);

--
-- Table structure for table meta__2P
--
CREATE TABLE meta__2P (
  pk_meta        serial      PRIMARY KEY,
  name           char(62)    NOT NULL UNIQUE CHECK (name <> ''),
  fk_name_l1b    char(62)    REFERENCES meta__1P(name),
  fileSize       integer     NOT NULL CHECK (fileSize > 0),
  receiveDate    timestamp   NOT NULL,
  procStage      proc_stage  NOT NULL,
  procCenter     char(6)     NOT NULL,
  softVersion    varchar(14) NOT NULL,
  fittingErrSum  quality     NOT NULL,
  dateTimeStart  timestamp   NOT NULL,
  muSecStart     integer     NOT NULL CHECK (muSecStart >= 0),
  dateTimeStop   timestamp   NOT NULL,
  muSecStop      integer     NOT NULL CHECK (muSecStop >= 0),
  absOrbit       integer     NOT NULL CHECK (absOrbit >= 0),
  relOrbit       smallint    NOT NULL,
  numDataSets    smallint    NOT NULL,
  nadirFittingWindows  varchar(192) NOT NULL,
  limbFittingWindows   varchar(192) NOT NULL,
  occultFittingWindows varchar(192) NOT NULL,
  numNadirFW     smallint    NOT NULL,
  numLimbFW      smallint    NOT NULL,
  numOccultFW    smallint    NOT NULL,
  noEntryDMOP    smallint    DEFAULT 0,
  delayedBy      real        
);
-- CREATE INDEX meta__2P_absOrbit      ON meta__2P (absOrbit);
-- CREATE INDEX meta__2P_dateTimeStart ON meta__2P (dateTimeStart);

--
-- Table structure for table "meta_fresco"
--
CREATE TABLE meta_fresco (
  pk_meta        serial      PRIMARY KEY,
  name           varchar(66) NOT NULL UNIQUE CHECK (name <> ''),
  fk_name_l1b    char(62)    REFERENCES meta__1P(name),
  fileSize       integer     NOT NULL CHECK (fileSize > 0),
  receiveDate    timestamp   NOT NULL,
  creationDate   timestamp   NOT NULL,
  dateTimeStart  timestamp   CHECK (numDataSets > 0 OR dateTimeStart IS NULL),
  muSecStart     integer     CHECK (numDataSets > 0 AND muSecStart >= 0 OR muSecStart IS NULL),
  dateTimeStop   timestamp   CHECK (numDataSets > 0 OR dateTimeStop IS NULL),
  muSecStop      integer     CHECK (numDataSets > 0 AND muSecStop >= 0 OR muSecStop IS NULL),
  softVersion    varchar(8)  ,
  fk_version_l1b varchar(10) ,
  absOrbit       integer     NOT NULL CHECK (absOrbit >= 0),
  numDataSets    smallint    NOT NULL
);
-- CREATE INDEX meta_fresco_absOrbit      ON meta_fresco (absOrbit);
-- CREATE INDEX meta_fresco_dateTimeStart ON meta_fresco (dateTimeStart);

--
-- Table structure for table "meta_imap_ch4"
--
CREATE TABLE meta_imap_ch4 (
  pk_meta          serial      PRIMARY KEY,
  name             varchar(66) NOT NULL UNIQUE CHECK (name <> ''),
  fk_name_l1b      char(62)    REFERENCES meta__1P(name),
  fileSize         integer     NOT NULL CHECK (fileSize > 0),
  receiveDate      timestamp   NOT NULL,
  creationDate     timestamp   NOT NULL,
  dateTimeStart    timestamp   CHECK (numDataSets > 0 OR dateTimeStart IS NULL),
  muSecStart       integer     CHECK (numDataSets > 0 AND muSecStart >= 0 OR muSecStart IS NULL),
  dateTimeStop     timestamp   CHECK (numDataSets > 0 OR dateTimeStop IS NULL),
  muSecStop        integer     CHECK (numDataSets > 0 AND muSecStop >= 0 OR muSecStop IS NULL),
  softVersion      varchar(8)  NOT NULL,
  absOrbit         integer     NOT NULL CHECK (absOrbit >= 0),
  numDataSets      smallint    NOT NULL
);
-- CREATE INDEX meta_imap_ch4_absOrbit      ON meta_imap_ch4 (absOrbit);
-- CREATE INDEX meta_imap_ch4_dateTimeStart ON meta_imap_ch4 (dateTimeStart);
--
-- Table structure for table "meta_imap_hdo"
--
CREATE TABLE meta_imap_hdo (
  pk_meta          serial      PRIMARY KEY,
  name             varchar(66) NOT NULL UNIQUE CHECK (name <> ''),
  fk_name_l1b      char(62)    REFERENCES meta__1P(name),
  fileSize         integer     NOT NULL CHECK (fileSize > 0),
  receiveDate      timestamp   NOT NULL,
  creationDate     timestamp   NOT NULL,
  dateTimeStart    timestamp   CHECK (numDataSets > 0 OR dateTimeStart IS NULL),
  muSecStart       integer     CHECK (numDataSets > 0 AND muSecStart >= 0 OR muSecStart IS NULL),
  dateTimeStop     timestamp   CHECK (numDataSets > 0 OR dateTimeStop IS NULL),
  muSecStop        integer     CHECK (numDataSets > 0 AND muSecStop >= 0 OR muSecStop IS NULL),
  softVersion      varchar(8)  NOT NULL,
  absOrbit         integer     NOT NULL CHECK (absOrbit >= 0),
  numDataSets      smallint    NOT NULL
);
-- CREATE INDEX meta_imap_hdo_absOrbit      ON meta_imap_hdo (absOrbit);
-- CREATE INDEX meta_imap_hdo_dateTimeStart ON meta_imap_hdo (dateTimeStart);

--
-- Table structure for table "meta_imlm_co"
--
CREATE TABLE meta_imlm_co (
  pk_meta          serial      PRIMARY KEY,
  name             varchar(66) NOT NULL UNIQUE CHECK (name <> ''),
  fk_name_l1b      char(62)    REFERENCES meta__1P(name),
  fileSize         integer     NOT NULL CHECK (fileSize > 0),
  receiveDate      timestamp   NOT NULL,
  creationDate     timestamp   NOT NULL,
  dateTimeStart    timestamp   CHECK (numDataSets > 0 OR dateTimeStart IS NULL),
  muSecStart       integer     CHECK (numDataSets > 0 AND muSecStart >= 0 OR muSecStart IS NULL),
  dateTimeStop     timestamp   CHECK (numDataSets > 0 OR dateTimeStop IS NULL),
  muSecStop        integer     CHECK (numDataSets > 0 AND muSecStop >= 0 OR muSecStop IS NULL),
  softVersion      varchar(8)  NOT NULL,
  pixelMaskVersion varchar(16) NOT NULL,
  cloudMaskVersion varchar(16) NOT NULL,
  absOrbit         integer     NOT NULL CHECK (absOrbit >= 0),
  windowPixel      integer[2]  NOT NULL,
  windowWave       real[2]     NOT NULL,
  numDataSets      smallint    NOT NULL
);
-- CREATE INDEX meta_imlm_co_absOrbit      ON meta_imlm_co (absOrbit);
-- CREATE INDEX meta_imlm_co_dateTimeStart ON meta_imlm_co (dateTimeStart);

--
-- Table structure for table "meta_imlm_h2o"
--
CREATE TABLE meta_imlm_h2o (
  pk_meta          serial      PRIMARY KEY,
  name             varchar(66) NOT NULL UNIQUE CHECK (name <> ''),
  fk_name_l1b      char(62)    REFERENCES meta__1P(name),
  fileSize         integer     NOT NULL CHECK (fileSize > 0),
  receiveDate      timestamp   NOT NULL,
  creationDate     timestamp   NOT NULL,
  dateTimeStart    timestamp   CHECK (numDataSets > 0 OR dateTimeStart IS NULL),
  muSecStart       integer     CHECK (numDataSets > 0 AND muSecStart >= 0 OR muSecStart IS NULL),
  dateTimeStop     timestamp   CHECK (numDataSets > 0 OR dateTimeStop IS NULL),
  muSecStop        integer     CHECK (numDataSets > 0 AND muSecStop >= 0 OR muSecStop IS NULL),
  softVersion      varchar(8)  NOT NULL,
  pixelMaskVersion varchar(16) NOT NULL,
  cloudMaskVersion varchar(16) NOT NULL,
  absOrbit         integer     NOT NULL CHECK (absOrbit >= 0),
  windowPixel      integer[2]  NOT NULL,
  windowWave       real[2]     NOT NULL,
  numDataSets      smallint    NOT NULL
);
-- CREATE INDEX meta_imlm_h2o_absOrbit      ON meta_imlm_h2o (absOrbit);
-- CREATE INDEX meta_imlm_h2o_dateTimeStart ON meta_imlm_h2o (dateTimeStart);

--
-- Table structure for table "meta_mcfs"
--
CREATE TABLE meta_mcfs (
  pk_meta          serial      PRIMARY KEY,
  name             varchar(66) NOT NULL UNIQUE CHECK (name <> ''),
  fk_name_l1b      char(62)    REFERENCES meta__1P(name),
  fileSize         integer     NOT NULL CHECK (fileSize > 0),
  receiveDate      timestamp   NOT NULL,
  creationDate     timestamp   NOT NULL,
  dateTimeStart    timestamp   CHECK (numDataSets > 0 OR dateTimeStart IS NULL),
  muSecStart       integer     CHECK (numDataSets > 0 AND muSecStart >= 0 OR muSecStart IS NULL),
  dateTimeStop     timestamp   CHECK (numDataSets > 0 OR dateTimeStop IS NULL),
  muSecStop        integer     CHECK (numDataSets > 0 AND muSecStop >= 0 OR muSecStop IS NULL),
  absOrbit         integer     NOT NULL CHECK (absOrbit >= 0),
  numDataSets      smallint    NOT NULL
);
-- CREATE INDEX meta_mcfs_absOrbit      ON meta_mcfs (absOrbit);
-- CREATE INDEX meta_mcfs_dateTimeStart ON meta_mcfs (dateTimeStart);

--
-- Table structure for table "meta_tosomi"
--
CREATE TABLE meta_tosomi (
  pk_meta        serial      PRIMARY KEY,
  name           varchar(66) NOT NULL UNIQUE CHECK (name <> ''),
  fk_name_l1b    char(62)    REFERENCES meta__1P(name),
  fileSize       integer     NOT NULL CHECK (fileSize > 0),
  receiveDate    timestamp   NOT NULL,
  creationDate   timestamp   NOT NULL,
  dateTimeStart  timestamp   CHECK (numDataSets > 0 OR dateTimeStart IS NULL),
  muSecStart     integer     CHECK (numDataSets > 0 AND muSecStart >= 0 OR muSecStart IS NULL),
  dateTimeStop   timestamp   CHECK (numDataSets > 0 OR dateTimeStop IS NULL),
  muSecStop      integer     CHECK (numDataSets > 0 AND muSecStop >= 0 OR muSecStop IS NULL),
  softVersion    varchar(8)  ,
  fk_version_l1b varchar(10) ,
  absOrbit       integer     NOT NULL CHECK (absOrbit >= 0),
  numDataSets    smallint    NOT NULL
);
-- CREATE INDEX meta_tosomi_absOrbit      ON meta_tosomi (absOrbit);
-- CREATE INDEX meta_tosomi_dateTimeStart ON meta_tosomi (dateTimeStart);

--
-- Table structure for table "stateinfo_meta__0P"
--
CREATE TABLE stateinfo_meta__0P (
  fk_stateinfo integer  REFERENCES stateinfo(pk_stateinfo),
  fk_meta      integer  REFERENCES meta__0P(pk_meta) ON DELETE CASCADE,
  numAUX       smallint NOT NULL,
  numDET       smallint NOT NULL,
  numPMD       smallint NOT NULL,
  delayedBy    real     NOT NULL,
  PRIMARY KEY (fk_stateinfo,fk_meta)
);

--
-- Table structure for table "stateinfo_meta__1P"
--
CREATE TABLE stateinfo_meta__1P (
  fk_stateinfo integer  REFERENCES stateinfo(pk_stateinfo),
  fk_meta      integer  REFERENCES meta__1P(pk_meta) ON DELETE CASCADE,
  delayedBy    real     NOT NULL,
  PRIMARY KEY  (fk_stateinfo,fk_meta)
);

--
-- Table structure for table "stateinfo_meta__2P"
--
CREATE TABLE stateinfo_meta__2P (
  fk_stateinfo integer  REFERENCES stateinfo(pk_stateinfo),
  fk_meta      integer  REFERENCES meta__2P(pk_meta) ON DELETE CASCADE,
  delayedBy    real     NOT NULL,
  PRIMARY KEY (fk_stateinfo, fk_meta)
);

--
-- Table structure for table "tileinfo"
--
CREATE TABLE tileinfo (
  pk_tileinfo     bigserial        PRIMARY KEY,
  julianDay       double precision NOT NULL UNIQUE,
  fk_stateinfo    integer          REFERENCES stateinfo(pk_stateinfo),
  integrationTime smallint         NOT NULL,
  pixelType       smallint         NOT NULL,
  positionESM     real             NOT NULL,
  satZenithAngle  real             NOT NULL,
  satAzimuthAngle real             NOT NULL,
  sunZenithAngle  real             NOT NULL,
  sunAzimuthAngle real             NOT NULL
);
-- CREATE INDEX tileinfo_julianDay ON tileinfo (julianDay);
-- SRID voor WGS84 is 4326
SELECT AddGeometryColumn( 'tileinfo', 'tile', 4326, 'POLYGON', 2 );
CREATE INDEX tileinfo_tile ON tileinfo USING GIST (tile);

--
-- Table structure for table "tile_fresco"
--
CREATE TABLE tile_fresco (
  pk_tile          bigserial PRIMARY KEY,
  julianDay        double precision NOT NULL UNIQUE,
  fk_meta          integer   REFERENCES meta_fresco(pk_meta) ON DELETE CASCADE,
  integrationTime  smallint  NOT NULL,
  errorFlag        smallint  NOT NULL,
  cloudFraction    real      NOT NULL,
  cloudTopHeight   real      NOT NULL,
  cloudTopPressure real      NOT NULL,
  cloudAlbedo      real      NOT NULL,
  surfaceHeight    real      NOT NULL,
  surfacePressure  real      NOT NULL,
  surfaceAlbedo    real      NOT NULL
);

--
-- Table structure for table "tile_imap_ch4"
--
CREATE TABLE tile_imap_ch4 (
  pk_tile          bigserial PRIMARY KEY,
  julianDay        double precision NOT NULL UNIQUE,
  fk_meta          integer   REFERENCES meta_imap_ch4(pk_meta) ON DELETE CASCADE,
  integrationTime  smallint  NOT NULL,
  meanElevation    real      NOT NULL,
  VCD_CH4          real      NOT NULL,
  VCD_CH4_ERROR    real      NOT NULL,
  VCD_CO2          real      NOT NULL,
  VCD_CO2_ERROR    real      NOT NULL,
  xVMR_CH4         real      NOT NULL
);

--
-- Table structure for table "tile_imap_hd0"
--
CREATE TABLE tile_imap_hdo (
  pk_tile          bigserial PRIMARY KEY,
  julianDay        double precision NOT NULL UNIQUE,
  fk_meta          integer   REFERENCES meta_imap_hdo(pk_meta) ON DELETE CASCADE,
  integrationTime  smallint  NOT NULL,
  meanElevation    real      NOT NULL,
  VCD_HDO          real      NOT NULL,
  VCD_HDO_ERROR    real      NOT NULL,
  VCD_H2O          real      NOT NULL,
  VCD_H2O_ERROR    real      NOT NULL,
  VCD_H2O_MODEL    real      NOT NULL
);

--
-- Table structure for table "tile_imlm_co"
--
CREATE TABLE tile_imlm_co (
  pk_tile          bigserial PRIMARY KEY,
  julianDay        double precision NOT NULL UNIQUE,
  fk_meta          integer   REFERENCES meta_imlm_co(pk_meta) ON DELETE CASCADE,
  integrationTime  smallint  NOT NULL,
  errorFlag        smallint  NOT NULL,
  meanElevation    real      NOT NULL,
  cloudFraction    real      ,
  surfaceAlbedo    real      ,
  VCD_CO           real      ,
  VCD_CO_ERROR     real      ,
  VCD_CH4          real      ,
  VCD_CH4_ERROR    real      
);

--
-- Table structure for table "tile_imlm_h2o"
--
CREATE TABLE tile_imlm_h2o (
  pk_tile          bigserial PRIMARY KEY,
  julianDay        double precision NOT NULL UNIQUE,
  fk_meta          integer   REFERENCES meta_imlm_h2o(pk_meta) ON DELETE CASCADE,
  integrationTime  smallint  NOT NULL,
  errorFlag        smallint  NOT NULL,
  meanElevation    real      NOT NULL,
  cloudFraction    real      ,
  surfaceAlbedo    real      ,
  VCD_H2O          real      ,
  VCD_H2O_ERROR    real      ,
  VCD_CH4          real      ,
  VCD_CH4_ERROR    real      
);

--
-- Table structure for table "tile_mcfs"
--
CREATE TABLE tile_mcfs (
  pk_tile            bigserial PRIMARY KEY,
  julianDay          double precision NOT NULL UNIQUE,
  fk_meta            integer   REFERENCES meta_mcfs(pk_meta) ON DELETE CASCADE,
  integrationTime    smallint  NOT NULL,
  quality            smallint  NOT NULL,
  numMerisPixels     smallint  NOT NULL,
  cloudFraction      real      NOT NULL,
  cloudFractionThick real      NOT NULL
);

--
-- Table structure for table "tile_tosomi"
--
CREATE TABLE tile_tosomi (
  pk_tile          bigserial PRIMARY KEY,
  julianDay        double precision NOT NULL UNIQUE,
  fk_meta          integer   REFERENCES meta_tosomi(pk_meta) ON DELETE CASCADE,
  integrationTime  smallint  NOT NULL,
  cloudFraction    smallint  NOT NULL,
  cloudTopPress    integer   NOT NULL,
  amf              real      NOT NULL,
  amfCloud         real      NOT NULL,
  ozone            real      NOT NULL,
  ozoneSlant       real      NOT NULL
);

--
-- Table structure for table "tile_cld_ol"
--
CREATE TABLE tile_cld_ol (
  pk_tile           bigserial PRIMARY KEY,
  julianDay         double precision NOT NULL UNIQUE,
  fk_meta           integer   REFERENCES meta__2P(pk_meta) ON DELETE CASCADE,
  cloudFraction     real      NOT NULL,
  cloudTopPress     real      NOT NULL,
  cloudOpticalDepth real      NOT NULL,
  cloudBRDF         real      NOT NULL,
  surfacePress      real      NOT NULL,
  surfaceRefl       real      NOT NULL,
  aerosolIndex      real      NOT NULL
);

--
-- Table structure for table "tile_no2_ol"
--
CREATE TABLE tile_no2_ol (
  pk_tile           bigserial PRIMARY KEY,
  julianDay         double precision NOT NULL UNIQUE,
  fk_meta           integer   REFERENCES meta__2P(pk_meta) ON DELETE CASCADE,
  verticalColumn    real      NOT NULL,
  slantColumn       real      NOT NULL,
  amfGround         real      NOT NULL,
  amfCloud          real      NOT NULL
);

--
-- Table structure for table "tile_o3_ol"
--
CREATE TABLE tile_o3_ol (
  pk_tile           bigserial PRIMARY KEY,
  julianDay         double precision NOT NULL UNIQUE,
  fk_meta           integer   REFERENCES meta__2P(pk_meta) ON DELETE CASCADE,
  verticalColumn    real   NOT NULL,
  slantColumn       real   NOT NULL,
  amfGround         real   NOT NULL,
  amfCloud          real   NOT NULL
);

--
-- Table structure for table "fresco_tileinfo"
--
CREATE TABLE fresco_tileinfo (
  fk_tile      bigint    REFERENCES tile_fresco(pk_tile) ON DELETE CASCADE,
  fk_tileinfo  bigint    REFERENCES tileinfo(pk_tileinfo),
  PRIMARY KEY (fk_tile, fk_tileinfo)
);

--
-- Table structure for table "tosomi_tileinfo"
--
CREATE TABLE tosomi_tileinfo (
  fk_tile      bigint    REFERENCES tile_tosomi(pk_tile) ON DELETE CASCADE,
  fk_tileinfo  bigint    REFERENCES tileinfo(pk_tileinfo),
  PRIMARY KEY (fk_tile, fk_tileinfo)
);

--
-- Table structure for table "mcfs_tileinfo"
--
CREATE TABLE mcfs_tileinfo (
  fk_tile       bigint   REFERENCES tile_mcfs(pk_tile) ON DELETE CASCADE,
  fk_tileinfo   bigint   REFERENCES tileinfo(pk_tileinfo),
  PRIMARY KEY (fk_tile, fk_tileinfo)
);

--
-- Table structure for table "imlm_co_tileinfo"
--
CREATE TABLE imlm_co_tileinfo (
  fk_tile       bigint   REFERENCES tile_imlm_co(pk_tile) ON DELETE CASCADE,
  fk_tileinfo   bigint   REFERENCES tileinfo(pk_tileinfo),
  PRIMARY KEY (fk_tile, fk_tileinfo)
);

--
-- Table structure for table "imlm_h2o_tileinfo"
--
CREATE TABLE imlm_h2o_tileinfo (
  fk_tile       bigint   REFERENCES tile_imlm_h2o(pk_tile) ON DELETE CASCADE,
  fk_tileinfo   bigint   REFERENCES tileinfo(pk_tileinfo),
  PRIMARY KEY (fk_tile, fk_tileinfo)
);

--
-- Table structure for table "imap_ch4_tileinfo"
--
CREATE TABLE imap_tileinfo (
  fk_tile       bigint   REFERENCES tile_imap_ch4(pk_tile) ON DELETE CASCADE,
  fk_tileinfo   bigint   REFERENCES tileinfo(pk_tileinfo),
  PRIMARY KEY (fk_tile, fk_tileinfo)
);

--
-- Table structure for table "imap_hdo_tileinfo"
--
CREATE TABLE imap_tileinfo (
  fk_tile       bigint   REFERENCES tile_imap_hdo(pk_tile) ON DELETE CASCADE,
  fk_tileinfo   bigint   REFERENCES tileinfo(pk_tileinfo),
  PRIMARY KEY (fk_tile, fk_tileinfo)
);

--
-- Table structure for table "cld_ol_tileinfo"
--
CREATE TABLE cld_ol_tileinfo (
  fk_tile       bigint   REFERENCES tile_cld_ol(pk_tile) ON DELETE CASCADE,
  fk_tileinfo   bigint   REFERENCES tileinfo(pk_tileinfo),
  PRIMARY KEY (fk_tile, fk_tileinfo)
);

--
-- Table structure for table "no2_ol_tileinfo"
--
CREATE TABLE no2_ol_tileinfo (
  fk_tile       bigint   REFERENCES tile_no2_ol(pk_tile) ON DELETE CASCADE,
  fk_tileinfo   bigint   REFERENCES tileinfo(pk_tileinfo),
  PRIMARY KEY (fk_tile, fk_tileinfo)
);

--
-- Table structure for table "o3_ol_tileinfo"
--
CREATE TABLE o3_ol_tileinfo (
  fk_tile       bigint   REFERENCES tile_o3_ol(pk_tile) ON DELETE CASCADE,
  fk_tileinfo   bigint   REFERENCES tileinfo(pk_tileinfo),
  PRIMARY KEY (fk_tile, fk_tileinfo)
);

--
-- Table structure for table "auxiliary"
--
CREATE TABLE auxiliary (
  pk_id        serial    PRIMARY KEY,
  fileName     char(62)  NOT NULL UNIQUE
);

GRANT SELECT ON auxiliary, auxiliary_pk_id_seq, cld_ol_tileinfo, fresco_tileinfo, imap_ch4_tileinfo, imap_hdo_tileinfo, imlm_co_tileinfo, imlm_h2o_tileinfo,
      mcfs_tileinfo, meta__0p, meta__0p_pk_meta_seq, meta__1p, meta__1p_pk_meta_seq, meta__2p,
      meta__2p_pk_meta_seq, meta_fresco, meta_fresco_pk_meta_seq, meta_imap_ch4, meta_imap_hdo, meta_imap_ch4_pk_meta_seq, meta_imap_hdo_pk_meta_seq,
      meta_imlm_co, meta_imlm_h2o, meta_imlm_co_pk_meta_seq, meta_imlm_h2o_pk_meta_seq, meta_mcfs, meta_mcfs_pk_meta_seq, meta_tosomi, 
      meta_tosomi_pk_meta_seq, no2_ol_tileinfo, o3_ol_tileinfo, stateinfo, stateinfo_meta__0p,
      stateinfo_meta__1p, stateinfo_meta__2p, stateinfo_pk_stateinfo_seq, tile_cld_ol, tile_cld_ol_pk_tile_seq,
      tile_fresco, tile_fresco_pk_tile_seq, tile_imap_ch4, tile_imap_hdo, tile_imap_ch4_pk_tile_seq, tile_imap_hdo_pk_tile_seq, tile_imlm_co, tile_imlm_h2o, tile_imlm_co_pk_tile_seq, tile_imlm_h2o_pk_tile_seq,  
      tile_mcfs, tile_mcfs_pk_tile_seq, tile_no2_ol, tile_no2_ol_pk_tile_seq, tile_o3_ol, 
      tile_o3_ol_pk_tile_seq, tile_tosomi, tile_tosomi_pk_tile_seq, tileinfo, tileinfo_pk_tileinfo_seq,
      tosomi_tileinfo TO nadc_user;
GRANT USAGE ON SCHEMA nadc_admin to nadc_user;
